@@ -662,9 +662,9 @@ int Zopen (
 
  if (check_cli_error("OPEN", dbcp) == 0)	return(0);
 
-  /* Fetch only until we have the PrepInfo parcel. */
+  /* Fetch only until we have the Success parcel. */
  status = 1;
- while (dbcp->fet_parcel_flavor != PclPREPINFO && status == 1) {
+ while (dbcp->fet_parcel_flavor != PclSUCCESS && status == 1) {
     status = _fetch_parcel("OPEN", dbcp, req);
  }
 
@@ -727,9 +727,9 @@ int Zopenseg (
  req->req_num = dbcp->o_req_id;
  dbcp->i_req_id = dbcp->o_req_id;
 
-  /* Fetch only until we have the DataInfo parcel. */
+  /* Fetch only until we have the Success parcel. */
  status = 1;
- while (dbcp->fet_parcel_flavor != PclDATAINFO && status == 1) {
+ while (dbcp->fet_parcel_flavor != PclSUCCESS && status == 1) {
     status = _fetch_parcel("OPENSEG", dbcp, req);
  }
 
@@ -855,7 +855,7 @@ int Zopenp (
 
   /* Fetch only until we have the PrepInfo parcel. */
  status = 1;
- while (dbcp->fet_parcel_flavor != PclPREPINFO && status == 1) {
+ while (dbcp->fet_parcel_flavor != PclSUCCESS && status == 1) {
     status = _fetch_parcel("OPENP", dbcp, req);
  }
 
@@ -920,9 +920,9 @@ int Zopenp_args (
  req->req_num = dbcp->o_req_id;
  dbcp->i_req_id = dbcp->o_req_id;
 
-  /* Fetch only until we have the PrepInfo parcel. */
+  /* Fetch only until we have the Success parcel. */
  status = 1;
- while (dbcp->fet_parcel_flavor != PclPREPINFO && status == 1) {
+ while (dbcp->fet_parcel_flavor != PclSUCCESS && status == 1) {
     status = _fetch_parcel("OPENP", dbcp, req);
  }
 
@@ -943,21 +943,31 @@ char * Zfetch (
  dbcp->change_opts = 'N';
  dbcp->i_req_id = req->req_num;
 
-  /* Read the record. If we find a DataInfo parcel, we need to keep
-     reading until we get a Record; if we find EndStatement, we keep
-     reading until we get EndRequest. */
+  /*------------------------------------------------------------------
+    Fetch the record. There are five possible parcels.
+    PrepInfo or DataInfo: keep reading until we get a Record.
+    Record: return it.
+    EndStatement: keep reading until the next Record or EndRequest.
+    EndRequest: return a null pointer ("end of file").
+    anything else: unexpected; return a null pointer.
+  **----------------------------------------------------------------*/
 
  status = 1;
 
  while (1) {
     status = _fetch_parcel("FETCH", dbcp, req);
-    if (status == 1 && dbcp->fet_parcel_flavor == PclRECORD)
-       return dbcp->fet_data_ptr;
-    if (status == 1 && (dbcp->fet_parcel_flavor == PclDATAINFO ||
-        dbcp->fet_parcel_flavor == PclENDSTATEMENT))
-       continue;
-    else
+    if (status == 1) {
+       if (dbcp->fet_parcel_flavor == PclRECORD)
+          return dbcp->fet_data_ptr;
+       if (dbcp->fet_parcel_flavor == PclPREPINFO ||
+           dbcp->fet_parcel_flavor == PclDATAINFO ||
+           dbcp->fet_parcel_flavor == PclENDSTATEMENT)
+          continue;
+       if (dbcp->fet_parcel_flavor == PclENDREQUEST)
+          return (char *) 0;
+    } else {
        return (char *) 0;
+    }
  }
 }
 
